#!/usr/bin/python3

# DatabaseTable by Luke Shiner (luke@lukeshiner.com)

from . databaseconnection import DatabaseConnection
from . databasecolumn import DatabaseColumn


class DatabaseTable(DatabaseConnection):
    """ Container used to provide usefull functions for accessing and
    manipulating a MySQL Table.
    """

    update_key = None
    no_update_columns = ()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.table = kwargs['table']
        self.description = self.query("DESCRIBE " + self.table)
        self.columns = []
        for column in self.description:
            self.columns.append(DatabaseColumn(column))

    def get_all(self):
        """ Returns all data in table as a two dimensional list.  """
        return self.result_to_list(self.query("SELECT * FROM " + self.table))

    def result_to_list(self, set_):
        """ Converts a two dimensional set to a two dimensional list.  """
        newArray = []
        for row in set_:
            newArray.append(list(row))
        return newArray

    def get_columns(self):
        """ Returns a list containing the table's column names.  """
        columnArray = []
        for column in self.columns:
            columnArray.append(column.name)
        return columnArray

    def truncate(self):
        self.query("TRUNCATE " + self.table)

    def replace_from_file(self, update_file):
        self.truncate()
        self.update_from_file(update_file)

    def get_update_from_file_query(self, update_file):
        """ Updates MySQL Table with data from a .csv or a Table object.
        If self.update_key is set any record with a matching value will be
        updated. file.header or the first row of the .csv must match the table
        column names (order is not important).
        Prints a running percentage complete.
        Returns the number of records updated, then the number new records
        inserted.
        """

        query = "INSERT INTO " + self.table + " "
        query += self.get_query_columns_string() + " VALUES "
        update_rows = []
        for row in update_file.rows:
            update_rows.append(self.get_insert_query_line(row))
        query += ', '.join(update_rows)
        if self.update_key is not None:
            query += " ON DUPLICATE KEY UPDATE "
            duplicate_updates = []
            for column in self.get_columns():
                if column != self.update_key:
                    duplicate_updates.append(
                        "`" + column + "`=VALUES(`" + column + "`)")
            query += ', '.join(duplicate_updates)
        query = query + ";"

        return query

    def update_from_file(self, update_file, split_rows=0):
        print("Updating " + self.database + "." + self.table + " from file")
        if split_rows > 0:
            tables = update_file.split_by_row_count(split_rows)
            queries = []
            for table in tables:
                queries.append(self.get_update_from_file_query(table))
            for query in queries:
                self.query(query)
        else:
            self.query(self.get_update_from_file_query(update_file))

    def get_column(self, column):
        """ Uses DatabaseConnection.get_column() to return a set
        containing all values in the specified column.
        """

        return super().get_column(self.table, column)

    def get_column_as_strings(self, column):
        return super().get_column_as_strings(self.table, column)

    def get_insert_query(self, row):
        query = "INSERT INTO " + self.table + " "
        query += self.get_query_columns_string()
        query += " VALUES "
        query += self.get_insert_query_line(row)
        return query

    def get_insert_query_line(self, row):
        """ Creates an apropriate insert query to insert data from a
        TableRow into the table.
        """

        values = self.get_query_values(row)
        query = "("
        query += ', '.join(values)
        query = query + ")"
        return query

    def get_update_query_line(self, row):
        columns = self.get_query_columns()
        values = self.get_query_values(row)
        query = " SET ("
        i = 0
        while i < len(columns):
            value_string = str(columns[i]) + "=" + str(values[i])
            query += ', '.join((query, value_string))
            i += 1
        query += ") WHERE " + self.update_key + "="
        query += self.escape_string(row.get_column(self.update_key))
        return query

    def get_update_query(self, row):
        """ Creates an apropriate update query to update a record with
        data from a TableRow.
        """

        columns = self.get_query_columns()
        values = self.get_query_values(row)
        query = "UPDATE " + self.table
        i = 0
        query += self.get_update_query_line(row) + ";"
        return query

    def get_query_columns(self):
        """ Returns a list of columns required for update and insert
        statements quoted with backticks.
        """

        backtick = '`'
        columns = []
        for column in self.columns:
            if column.name not in self.no_update_columns:
                columns.append(backtick + str(column.name) + backtick)
        return columns

    def get_query_columns_string(self):
        string = '('
        columns = self.get_query_columns()
        string += ', '.join(columns)
        string += ")"
        return string

    def get_query_values(self, row):
        """ Properly formats data from a TableRow for use in a MySQL
        insert or update query.
        """

        values = []
        for column in self.columns:
            if column.name not in self.no_update_columns:
                record = row.get_column(column.name)
                if record == '' or record is None:
                        record = 'NULL'
                elif column.data_type in ('varchar', 'text'):
                    record = "'" + self.escape_string(record) + "'"
                values.append(record)
        return values

    def truncate(self):
        """ This will empty the table. Data may not be recoverable.  """
        self.query("TRUNCATE " + self.table)

    def to_table(self):
        """ Returns Table containg this table's data.  """
        try:
            from .. table . table import Table as Table
        except ImportError:
            print('Table module not found')
            return None
        table = Table()
        table.load_from_database_table(self)
        return table

    def write(self, filepath):
        """ Creates a .csv of the table's data with the specified filepath."""
        self.to_table().write(filepath)



from . databaseconnection import DatabaseConnection
from . databasetable import DatabaseTable


def connect(**kwargs):
    database_connection = DatabaseConnection(**kwargs)
    return database_connection


def get_table(**kwargs):
    database_table = DatabaseTable(**kwargs)
    return database_table



#!/usr/bin/python3

# DatabaseColumn by Luke Shiner (luke@lukeshiner.com)


class DatabaseColumn():
    """ Container for columns of data imported from a mysql Database.
    Used by DatabaseTable
    """

    def __init__(self, column_description):
        self.name = None
        self.data_type = None
        self.null = None
        self.name = column_description[0]
        self.data_type = self.get_data_type(column_description[1])

        if column_description[2] == 'NO':
            self.null = False
        elif column_description[2] == 'YES':
            self.null = True

    def get_data_type(self, dtype):
        """ Converts data type from mysql table description to a usable format.
        Used to properly format query statements in DatabaseTable
        """

        datatypes = ['varchar', 'int', 'text', 'float']
        for datatype in datatypes:
            if dtype[0:len(datatype)] == datatype:
                return datatype



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='pydatabase',
    version='1.0.02',
    description='Simple interface for MySQL databases',
    author='Luke Shiner',
    author_email='luke@lukeshiner.com',
    url='http://pydatabase.lukeshiner.com',
    keywords=['database', 'mysql', 'simple'],
    install_requires=['tabler', 'pymysql'],
    packages=find_packages())



#!/usr/bin/python3

# DatabaseTable by Luke Shiner (luke@lukeshiner.com)


class Query():
    """ Container used to provide usefull functions for accessing and
    manipulating a MySQL Table.
    """

    def __init__(self, **kwargs):
        pass

    def test_list_match(self, item_list):
        if not isinstance(item_list, list):
            raise TypeError("Can only test list object")
        list_type = type(item_list[0])
        for item in item_list:
            if not isinstance(item, list_type):
                return False
        return True

    def enforce_list_match(self, item_list):
        if not isinstance(item_list, list):
            raise TypeError("Can only test list object")
        list_type = type(item_list[0])
        for item in item_list:
            if not isinstance(item, list_type):
                raise TypeError("List must contain only on type")
                return False
        return True

    def get_where_clause(self, where):
        if isinstance(where, str):
            where_clause = where
        elif isinstance(where, dict):
            where_clause = "WHERE " + ' AND '.join(
                "`" + field + "`" + "=" + where[field] for field in where
            )
        else:
            where_clause = ''
        return where_clause



#!/usr/bin/python3

# DatabaseConnection by Luke Shiner (luke@lukeshiner.com)

import pymysql


class DatabaseConnection(object):
    """ Used as an interface with a local MySQL instance
    Requires pymysql (https://pypi.python.org/pypi/PyMySQL#downloads)
    """

    host = 'localhost'
    user = 'axevalley'
    password = 'fred'
    charset = 'utf8'

    def __init__(self, **kwargs):
        self.database = kwargs['database']
        if 'host' in kwargs:
            self.host = kwargs['host']
        if 'user' in kwargs:
            self.user = kwargs['user']
        if 'password' in kwargs:
            self.password = kwargs['password']
        if 'charset' in kwargs:
            self.charset = kwargs['charsest']

    def query(self, query):
        """ Sends query to MySQL database and returns query results.  """

        conn = pymysql.connect(host=self.host, user=self.user,
                               password=self.password, db=self.database,
                               charset=self.charset)
        cur = conn.cursor()
        try:
            cur.execute(str(query))
        except:
            print('Query Error: ')
            print(str(query))
            conn.close()
            return None
        conn.commit()
        results = cur.fetchall()
        cur.close()
        conn.close()
        return results

    def get_column(self, table, column):
        """Queries MySQL database for specified column from specified table and
        returns the data therein as a set.  """

        results = self.query("SELECT " + column + " FROM " + table)
        column = []
        for record in results:
            column.append(record[0])
        return set(column)

    def get_column_as_strings(self, table, column):
        """Queries MySQL database for specified column from specified table and
        returns the data therein as a set of strings.  """

        results = self.query("SELECT " + column + " FROM " + table)
        column = []
        for record in results:
            column.append(str(record[0]))
        return set(column)

    def escape_string(self, string):
        """Provides basic string escapes for single quote characters.  """

        newstring = str(string)
        newstring = newstring.replace("'", "\\'")
        return newstring



#!/usr/bin/python3

# SelectQuery by Luke Shiner (luke@lukeshiner.com)

from . query import Query


class SelectQuery(Query):
    """ Container used to provide usefull functions for accessing and
    manipulating a MySQL Table.
    """

    def __init__(
            self, database, table, fields, where=None, sort=None, limit=None):
        self.database = database
        self.table = table
        self.fields = fields
        self.where_clause = where
        self.sort_clause = sort
        self.limit = limit

    def __str__(self):
        query = "SELECT "
        query += ', '.join('`' + field + '`' for field in self.fields)
        query += " FROM " + self.database + "." + self.table
        query += " " + str(self.get_where_clause(self.where_clause))
        query += " " + str(self.get_sort_clause(self.sort_clause))
        if self.limit is not None:
            query += " LIMIT " + str(self.limit)
        query += ";"
        return query

    def get_sort_clause(self, sort):
        if sort is None:
            return ''
        sort_clause = "ORDER BY "
        if isinstance(sort, str):
            sort_clause += "`" + sort + "`"
        elif isinstance(sort, list):
            if isinstance(sort[0], str):
                sort_clause += ', '.join(
                    "`" + field + "`" for field in sort
                )
            elif isinstance(sort[0], list):
                clauses = []
                for clause in sort:
                    string = "`" + clause[0] + "`"
                    if len(clause) > 1:
                        string += " " + clause[1]
                    clauses.append(string)
                sort_clause += ', '.join(clauses)
        else:
            raise TypeError("sort must be string or list")
        return sort_clause
